C----------------------------------------------------------------------
c..
c..   readslha.f
c..
c..   needs 
c..   common-slha.f
c..
c..   NOTE: maximum length of input lines is currently 
c..         set to 200 characters.
c..
c..   To add a block:
c..   (1) add an array for its value in common-slha.f: NEWARRAY(100)
c..   (2) possibly add an array for the names of its entries in 
c..       common-slha.f:  CNEWARRAY(100)
c..   (3) add a section in subroutine READDATA that fills NEWARRAY.
c..   (4) add a section in subroutine SLHAMAP that fills CNEWARRAY.
c..
C----------------------------------------------------------------------

com-- 
com-- following subroutines from bbh@nnlo by Robert Harlander (see arXiv:hep-ph/0304035)
com-- 


C-{{{ subroutine readblocks:

      subroutine readblocks_pt(iunit,blocks,nkeys)
c..
c..   This is just an interface to readslha to read multiple blocks.
c..   
c..   Example: 
c..   BLOCKS = ('mass','sminputs','minpar')
c..
      implicit real*8(a-h,o-z)
      integer nkeys(*)
      character(*) blocks(*)

      i=1
      do while (blocks(i).ne.' ')
         call readslha_pt(iunit,blocks(i),ifound)
         nkeys(i) = ifound
         i=i+1
      enddo

      end

C-}}}
C-{{{ subroutine readslha:

      subroutine readslha_pt (iunit,blocktype,ifound)
c..
c..   Read block BLOCKTYPIN from SLHA input file
c..   and fill the corresponding COMMON block.
c..   SLHA input file must be open in unit IUNIT.
c..
c..   Example:
c..   BLOCKTYPIN = 'mass'
c..   
c..   If BLOCK is found in input file, IFOUND is set to 1.
c..   IFOUND = 0 otherwise.
c..   
      integer klen
      character cline*200,clineout*200,emptyline*200
      character blocktype*15
      character uppercase*1

      rewind(iunit)

      write(emptyline,1002)
      cline = emptyline
      lenblock = lnblnk(blocktype)

c..   change input to upper case:
      do j=1,lenblock
         blocktype(j:j)=uppercase(blocktype(j:j))
      enddo

      lcount=0
      iread=0
      ifound=0
c..   I use a GOTO loop, because I don't know any other way
c..   how to read to the end of a file without knowing its length.
 100  lcount = lcount+1
      read(iunit,1001,END=101) cline
      call normalline(cline,clineout)
c..   
c..   cline:    original input with comments and multiple blanks removed
c..   clineout: uppercase of cline
c..   
      cline = clineout
      do i=1,200
         clineout(i:i) = uppercase(clineout(i:i))
      enddo
c..   check if line is valid input line:
      if ((clineout(1:1).ne.' ').and.(clineout(1:1).ne.'#').and.
     &     (clineout(1:1).ne.'B').and.(clineout(1:1).ne.'D')) then
         write(6,*) '*** Error reading SLHA input file (UNIT=',iunit
     &        ,'):'
         write(6,*) 'Line ',lcount,': ',
     &        clineout(1:1),' = char(',
     &        ichar(clineout(1:1)),') not allowed in first column'
         stop
      endif
      if (clineout.eq.emptyline) goto 100
c..   If we are looking at a keyword (BLOCK, DECAY), then either
c..   - start reading data if it's the correct BLOCK
c..   - stop reading if we are already reading data
      if ((clineout(1:5).eq.'BLOCK').or.(clineout(1:5).eq.'DECAY')) then
         klen = 7
         do while (clineout(klen:klen).ne.' ') 
            klen=klen+1
         enddo
         klen=klen-1
         iread = 0
         if ((klen.eq.lenblock+6).and.
     &        (clineout(1:klen).eq.'BLOCK '//blocktype(1:lenblock)))
     &        then
            ifound = 1
            iread = 1
            goto 100
         endif
      endif
      if (iread.eq.1) then
         call slhablocks_pt(blocktype,cline,ierr)
         if (ierr.eq.1) then
            write(6,*) '*** Error reading SLHA input file (UNIT
     &           =',iunit,'):'
            write(6,*) 'Line ',lcount,':>',cline(1:60)
            stop
         endif
      endif
      goto 100

 1001 format(200a)
 1002 format(200(' '))
 101  continue
      end

C-}}}
C-{{{ subroutine normalline:

c$$$      subroutine normalline(clinein,clineout)
c$$$c..
c$$$c..   Remove multiple blanks, remove comments,
c$$$c..   and change everything to upper case
c$$$c..
c$$$c..   Example:
c$$$c..   clinein = '  some    string with   tabs and  #  comments'
c$$$c..   
c$$$c..   Returns:
c$$$c..   clineout = ' some string with tabs and     '
c$$$c..   
c$$$c..   The output string is always 200 chars long (trailing blanks)
c$$$c..   
c$$$      implicit real*8 (a-h,o-z)
c$$$      character cline*200,clinein*200,clineinv*200,clineout*200
c$$$      logical nonempty
c$$$
c$$$      cline = clinein
c$$$
c$$$c..   remove comments, change tabs to spaces, and change to upper case:
c$$$      do i=1,200
c$$$         if (cline(i:i).eq.char(9)) cline(i:i) = ' '
c$$$         if (cline(i:i).eq.char(13)) cline(i:i) = ' '
c$$$         if (cline(i:i).eq.'#') then
c$$$            do j=i,200
c$$$               cline(j:j) = ' '
c$$$            enddo
c$$$         endif
c$$$      enddo
c$$$
c$$$c..   remove multiple whitespace:
c$$$      do i=1,200
c$$$         clineinv(i:i) = cline(200+1-i:200+1-i)
c$$$      enddo
c$$$      clineout=' '
c$$$      i=1
c$$$      nonempty = .false.
c$$$      do while (i.le.200)
c$$$         do while ((clineinv(i:i).ne.' ').and.(i.le.200))
c$$$            nonempty = .true.
c$$$            Clineout = clineinv(i:i)//clineout
c$$$            i=i+1
c$$$         enddo
c$$$         if (i.le.200) then
c$$$            clineout = ' '//clineout
c$$$            i=i+1
c$$$         endif
c$$$         do while ((clineinv(i:i).eq.' ').and.(i.le.200))
c$$$            i=i+1
c$$$         enddo
c$$$      enddo
c$$$
c$$$      end

C-}}}
C-{{{ function uppercase:

c$$$      function uppercase(str)
c$$$
c$$$      character*1 str,uppercase
c$$$
c$$$      do ich=97,122
c$$$         if (char(ich).eq.str) then
c$$$            uppercase=char(ich-32)
c$$$            return
c$$$         else
c$$$            uppercase=str
c$$$         endif
c$$$      enddo
c$$$
c$$$      end
      
C-}}}


C-{{{ subroutine slhablocks:

      subroutine slhablocks_pt(blocktype,cline,ierr)
com-- 
com-- subroutine from bbh@nnlo by Robert Harlander (see arXiv:hep-ph/0304035)
com-- used MoReSusHi Block added
com-- 
c..
c..   Read the data in string CLINE according to the format
c..   of block BLOCKTYPE.
c..   Note that BLOCKTYPE must ALWAYS be in capital letters, and
c..   of length *15 (trailing blanks).
c..
c..   Here the COMMON blocks are actually filled with data.
c..
c..   Example:
c..   BLOCKTYPE = 'MASS           '
c..   CLINE = ' 25  1.14e02  '
c..
c..   Then smassbo(25) would be given the value 1.14e02.
c..   In other words, M_h = 114 GeV.
c..   For the particle codes, see SUBROUTINE SLHAMAP.
c..
c..   Note that no checking of the correct format of CLINE is done.
c..   
      implicit real*8(a-h,o-z)
      real*8 rval
      character blocktype*15,cline*200
      include 'Commons/common-slha.f'

      ierr=0

      if    (blocktype .eq. 'SMINPUTS  ') then
         read(cline,*,ERR=510) i,rval
         SMinput_SET(i) = .true.
         SMinput(i)  = rval

com-- Block added -{
      ELSEIF(blocktype .eq. 'MORESUSHIKEYS') THEN 
         read(cline,*,ERR=510) i,j
         MoReSusHi_key_SET(i) = .true.
         MoReSusHi_key(i)  = j
com-- }- Block added

com-- Block added -{
      ELSEIF(blocktype .eq. 'DISTRIB2') THEN 
         read(cline,*,ERR=510) i,rval
         distrib2_SET(i) = .true.
         distrib2(i)  = rval
com-- }- Block added

com-- Block added -{
      ELSEIF(blocktype .eq. 'SUSHI') THEN 
         read(cline,*,ERR=510) i,rval
         SusHi_SET(i) = .true.
         SusHi_var(i)  = rval
com-- }- Block added

com-- Block added -{
      ELSEIF(blocktype .eq. 'SCALES') THEN 
         read(cline,*,ERR=510) i,rval
         Scales_SET(i) = .true.
         Scales_var(i)  = rval
com-- }- Block added

com-- Block added -{
      ELSEIF(blocktype .eq. 'PDFSPEC') THEN 
         read(cline,*,ERR=510) i
         IF(i < 4) THEN
            read(cline,*,ERR=510) i,PDF_str(i)
            PDF_SET(i) = .true.
         ELSE
            read(cline,*,ERR=510) i,j
            PDF_SET(i) = .true.
            PDF_int(i)  = j
         ENDIF
com-- }- Block added

com-- Block added -{
      ELSEIF(blocktype .eq. 'MORESUSHI') THEN 
         read(cline,*,ERR=510) i
         IF(i == 6) THEN
            read(cline,*,ERR=510) i,j
            MoReSusHi_SET(i) = .true.
            MoReSusHi_int(i) = j
         ELSEIF(i > 6 .and. i < 10) THEN
            read(cline,*,ERR=510) i,MoReSusHi_str(i)
            MoReSusHi_SET(i) = .true.
         ELSEIF(i == 10) THEN
            read(cline,*,ERR=510) i,j
            MoReSusHi_SET(i) = .true.
            MoReSusHi_int(i) = j
         ELSEIF(i > 10 .and. i < 14) THEN
            read(cline,*,ERR=510) i,MoReSusHi_str(i)
            MoReSusHi_SET(i) = .true. 
         ELSEIF(i > 18 .and. i < 23) THEN
            read(cline,*,ERR=510) i,j
            MoReSusHi_SET(i) = .true.
            MoReSusHi_int(i) = j
         ELSE
            read(cline,*,ERR=510) i,rval
            MoReSusHi_SET(i) = .true.
            MoReSusHi_var(i)  = rval
         ENDIF
com-- }- Block added

      elseif (blocktype.eq.'MASS      ') then
         read(cline,*,ERR=510) i,rval
         if (i.le.100) then
            massBo_SET(i) = .true.
            massBo(i)  = rval
         elseif (i.lt.2000000) then
            lsmassfe1(i-1000000) = .true.
            smassfe1(i-1000000)  = rval
         else 
            lsmassfe2(i-2000000) = .true.
            smassfe2(i-2000000)  = rval
         endif

c$$$      elseif (blocktype.eq.'CREIN     ') then
c$$$         read(cline,*,ERR=510) i,rval
c$$$         if (i.le.10) then
c$$$            lslhacrein(i) = .true.
c$$$            slhacrein(i)  = rval
c$$$         endif
c$$$
c$$$      elseif (blocktype.eq.'HNNLO1    ') then
c$$$         read(cline,*,ERR=510) i
c$$$         if (i.eq.4) then
c$$$            read(cline,*,ERR=510) i,slhachar(i)
c$$$            lslhachar(i) = .true.
c$$$         else
c$$$            read(cline,*,ERR=510) i,j
c$$$            lhiggsnnlo1(i) = .true.
c$$$            higgsnnlo1(i)  = j
c$$$         endif
c$$$
c$$$      elseif (blocktype.eq.'HNNLO2    ') then
c$$$         read(cline,*,ERR=510) i,rval
c$$$         lhiggsnnlo2(i) = .true.
c$$$         higgsnnlo2(i)  = rval

      else
         write(6,*) 'Modify slha-func.f for BLOCK ',blocktype
         write(6,*) 'Stopped in <slha-func.f>.'
         stop
      endif
      
      return

 510  ierr=1
      
      end

C-}}}
